precision highp float;

uniform sampler2D inputImageTexture;
uniform vec2 singleStepOffset;
uniform highp vec4 params;

varying highp vec2 textureCoordinate;

const highp vec3 W = vec3(0.299,0.587,0.114);
const mat3 saturateMatrix = mat3(
		1.1102,-0.0598,-0.061,
		-0.0774,1.0826,-0.1186,
		-0.0228,-0.0228,1.1772);

float hardlight(float color)
{
	if(color <= 0.5)
	{
		color = color * color * 2.0;
	}
	else
	{
		color = 1.0 - ((1.0 - color)*(1.0 - color) * 2.0);
	}
	return color;
}

void main(){
	vec2 blurCoordinates[24];

	blurCoordinates[16] = textureCoordinate.xy + singleStepOffset * vec2(-4.0, -4.0);
	blurCoordinates[17] = textureCoordinate.xy + singleStepOffset * vec2(-4.0, 4.0);
	blurCoordinates[18] = textureCoordinate.xy + singleStepOffset * vec2(4.0, -4.0);
	blurCoordinates[19] = textureCoordinate.xy + singleStepOffset * vec2(4.0, 4.0);

	blurCoordinates[20] = textureCoordinate.xy + singleStepOffset * vec2(-2.0, -2.0);
	blurCoordinates[21] = textureCoordinate.xy + singleStepOffset * vec2(-2.0, 2.0);
	blurCoordinates[22] = textureCoordinate.xy + singleStepOffset * vec2(2.0, -2.0);
	blurCoordinates[23] = textureCoordinate.xy + singleStepOffset * vec2(2.0, 2.0);

	float sampleColor = texture2D(inputImageTexture, textureCoordinate).g * 12.0;

	sampleColor += texture2D(inputImageTexture, blurCoordinates[16]).g * 1.0;
	sampleColor += texture2D(inputImageTexture, blurCoordinates[17]).g * 1.0;
	sampleColor += texture2D(inputImageTexture, blurCoordinates[18]).g * 1.0;
	sampleColor += texture2D(inputImageTexture, blurCoordinates[19]).g * 1.0;

	sampleColor += texture2D(inputImageTexture, blurCoordinates[20]).g * 2.0;
	sampleColor += texture2D(inputImageTexture, blurCoordinates[21]).g * 2.0;
	sampleColor += texture2D(inputImageTexture, blurCoordinates[22]).g * 2.0;
	sampleColor += texture2D(inputImageTexture, blurCoordinates[23]).g * 2.0;

	sampleColor = sampleColor / 24.0;

	vec3 centralColor = texture2D(inputImageTexture, textureCoordinate).rgb;

	float highpass = centralColor.g - sampleColor + 0.5;

	for(int i = 0; i < 5;i++)
	{
		highpass = hardlight(highpass);
	}
	float lumance = dot(centralColor, W);

	float alpha = pow(lumance, params.r);

	vec3 smoothColor = centralColor + (centralColor-vec3(highpass))*alpha*0.1;

	smoothColor.r = clamp(pow(smoothColor.r, params.g),0.0,1.0);
	smoothColor.g = clamp(pow(smoothColor.g, params.g),0.0,1.0);
	smoothColor.b = clamp(pow(smoothColor.b, params.g),0.0,1.0);

	vec3 lvse = vec3(1.0)-(vec3(1.0)-smoothColor)*(vec3(1.0)-centralColor);
	vec3 bianliang = max(smoothColor, centralColor);
	vec3 rouguang = 2.0*centralColor*smoothColor + centralColor*centralColor - 2.0*centralColor*centralColor*smoothColor;

	gl_FragColor = vec4(mix(centralColor, lvse, alpha), 1.0);
	gl_FragColor.rgb = mix(gl_FragColor.rgb, bianliang, alpha);
	gl_FragColor.rgb = mix(gl_FragColor.rgb, rouguang, params.b);

	vec3 satcolor = gl_FragColor.rgb * saturateMatrix;
	gl_FragColor.rgb = mix(gl_FragColor.rgb, satcolor, params.a);

	gl_FragColor  = vec4((gl_FragColor.rgb + vec3(0.1)), gl_FragColor.w);
}